#include "stdafx.h"
#include "OcctWindow.h"


IMPLEMENT_STANDARD_RTTIEXT(OcctWindow, Aspect_Window)


OcctWindow::OcctWindow(IFWin *window)
{

	myXLeft = 0;
	myYTop = 0;
	myXRight = 1;
	myYBottom = 1;

	if (window == NULL)
	{
		return;
	}
	pWin = window;

	DMH::Rect  cRect = pWin->GetRect();
	myXLeft = cRect.left();
	myYTop = cRect.top();
	myXRight = cRect.right();
	myYBottom = cRect.bottom();
}


OcctWindow::~OcctWindow()
{
}

//! Opens the window <me>.
void OcctWindow::Map() const
{
	pWin->Show(true);
}

//! Closes the window <me>.
void OcctWindow::Unmap() const
{
	pWin->Show(false);
}

//! Apply the resizing to the window <me>.
Aspect_TypeOfResize OcctWindow::DoResize() const
{
	int                 aMask = 0;
	Aspect_TypeOfResize aMode = Aspect_TOR_UNKNOWN;

	if (pWin->IsShown())
	{
		DMH::Rect  cRect = pWin->GetRect();


		if (Abs(cRect.left() - myXLeft) > 2) aMask |= 1;
		if (Abs(cRect.right() - myXRight) > 2) aMask |= 2;
		if (Abs(cRect.top() - myYTop) > 2) aMask |= 4;
		if (Abs(cRect.bottom() - myYBottom) > 2) aMask |= 8;

		switch (aMask)
		{
		case 0:
			aMode = Aspect_TOR_NO_BORDER;
			break;
		case 1:
			aMode = Aspect_TOR_LEFT_BORDER;
			break;
		case 2:
			aMode = Aspect_TOR_RIGHT_BORDER;
			break;
		case 4:
			aMode = Aspect_TOR_TOP_BORDER;
			break;
		case 5:
			aMode = Aspect_TOR_LEFT_AND_TOP_BORDER;
			break;
		case 6:
			aMode = Aspect_TOR_TOP_AND_RIGHT_BORDER;
			break;
		case 8:
			aMode = Aspect_TOR_BOTTOM_BORDER;
			break;
		case 9:
			aMode = Aspect_TOR_BOTTOM_AND_LEFT_BORDER;
			break;
		case 10:
			aMode = Aspect_TOR_RIGHT_AND_BOTTOM_BORDER;
			break;
		default:
			break;
		}  // end switch

		*((Standard_Integer*)&myXLeft) = cRect.left();
		*((Standard_Integer*)&myXRight) = cRect.right();
		*((Standard_Integer*)&myYTop) = cRect.top();
		*((Standard_Integer*)&myYBottom) = cRect.bottom();
	}

	return aMode;
}

//! Apply the mapping change to the window <me>.
//! and returns TRUE if the window is mapped at screen.
Standard_Boolean OcctWindow::DoMapping() const { return pWin->IsShown(); }


//! Returns True if the window <me> is opened
//! and False if the window is closed.
Standard_Boolean OcctWindow::IsMapped() const { return pWin->IsShown(); }

//! Returns The Window RATIO equal to the physical
//! WIDTH/HEIGHT dimensions
Quantity_Ratio OcctWindow::Ratio() const
{
	DMH::Rect  cRect = pWin->GetRect();
	return Quantity_Ratio(cRect.right() - cRect.left()) / Quantity_Ratio(cRect.bottom() - cRect.top());

}

//! Returns The Window POSITION in PIXEL
void OcctWindow::Position(Standard_Integer& theX1, Standard_Integer& theY1, Standard_Integer& theX2, Standard_Integer& theY2) const
{
	DMH::Rect  cRect = pWin->GetRect();
	theX1 = cRect.left();
	theY1 = cRect.top();
	theX2 = cRect.right();
	theY2 = cRect.bottom();

}

//! Returns The Window SIZE in PIXEL
void OcctWindow::Size(Standard_Integer& theWidth, Standard_Integer& theHeight) const
{
	DMH::Rect  cRect = pWin->GetRect();
	theWidth = cRect.w;
	theHeight = cRect.h;
}

//! Returns native Window handle (HWND on Windows, Window with Xlib, and so on)
Aspect_Drawable OcctWindow::NativeHandle() const
{
	return (Aspect_Drawable)pWin->GetID();
}

//! Returns parent of native Window handle (HWND on Windows, Window with Xlib, and so on)
Aspect_Drawable OcctWindow::NativeParentHandle() const
{
	if (pWin->GetParentID() != NULL)
	{
		return (Aspect_Drawable)pWin->GetParent()->GetID();
	}

	return NULL;
}

//! Returns native Window FB config (GLXFBConfig on Xlib)
Aspect_FBConfig OcctWindow::NativeFBConfig() const { return NULL; }