#include "stdafx.h"
#include "cMainApp.h"


char szErrore[1024];
long NumErrore;

static void error_callback(int error, const char* description)
{
	fprintf_s(stderr, "Error %d: %s\n", error, description);
	sprintf_s(szErrore, "Error %d: %s\n", error, description);
	NumErrore = error;
}




TopoDS_Shape
MakeBottle(const Standard_Real myWidth, const Standard_Real myHeight,
	const Standard_Real myThickness)
{
	// Profile : Define Support Points
	gp_Pnt aPnt1(-myWidth / 2., 0, 0);
	gp_Pnt aPnt2(-myWidth / 2., -myThickness / 4., 0);
	gp_Pnt aPnt3(0, -myThickness / 2., 0);
	gp_Pnt aPnt4(myWidth / 2., -myThickness / 4., 0);
	gp_Pnt aPnt5(myWidth / 2., 0, 0);

	// Profile : Define the Geometry
	Handle(Geom_TrimmedCurve) anArcOfCircle = GC_MakeArcOfCircle(aPnt2, aPnt3, aPnt4);
	Handle(Geom_TrimmedCurve) aSegment1 = GC_MakeSegment(aPnt1, aPnt2);
	Handle(Geom_TrimmedCurve) aSegment2 = GC_MakeSegment(aPnt4, aPnt5);

	// Profile : Define the Topology
	TopoDS_Edge anEdge1 = BRepBuilderAPI_MakeEdge(aSegment1);
	TopoDS_Edge anEdge2 = BRepBuilderAPI_MakeEdge(anArcOfCircle);
	TopoDS_Edge anEdge3 = BRepBuilderAPI_MakeEdge(aSegment2);
	TopoDS_Wire aWire = BRepBuilderAPI_MakeWire(anEdge1, anEdge2, anEdge3);

	// Complete Profile
	gp_Ax1 xAxis = gp::OX();
	gp_Trsf aTrsf;

	aTrsf.SetMirror(xAxis);
	BRepBuilderAPI_Transform aBRepTrsf(aWire, aTrsf);
	TopoDS_Shape aMirroredShape = aBRepTrsf.Shape();
	TopoDS_Wire aMirroredWire = TopoDS::Wire(aMirroredShape);

	BRepBuilderAPI_MakeWire mkWire;
	mkWire.Add(aWire);
	mkWire.Add(aMirroredWire);
	TopoDS_Wire myWireProfile = mkWire.Wire();

	// Body : Prism the Profile
	TopoDS_Face myFaceProfile = BRepBuilderAPI_MakeFace(myWireProfile);
	gp_Vec aPrismVec(0, 0, myHeight);
	TopoDS_Shape myBody = BRepPrimAPI_MakePrism(myFaceProfile, aPrismVec);

	// Body : Apply Fillets
	BRepFilletAPI_MakeFillet mkFillet(myBody);
	TopExp_Explorer anEdgeExplorer(myBody, TopAbs_EDGE);
	while (anEdgeExplorer.More()) {
		TopoDS_Edge anEdge = TopoDS::Edge(anEdgeExplorer.Current());
		//Add edge to fillet algorithm
		mkFillet.Add(myThickness / 24., anEdge);
		anEdgeExplorer.Next();
	}

	myBody = mkFillet.Shape();

	// Body : Add the Neck	
	gp_Pnt neckLocation(0, 0, myHeight);
	gp_Dir neckAxis = gp::DZ();
	gp_Ax2 neckAx2(neckLocation, neckAxis);

	Standard_Real myNeckRadius = myThickness / 4.;
	Standard_Real myNeckHeight = myHeight / 10.;

	BRepPrimAPI_MakeCylinder MKCylinder(neckAx2, myNeckRadius, myNeckHeight);
	TopoDS_Shape myNeck = MKCylinder.Shape();

	myBody = BRepAlgoAPI_Fuse(myBody, myNeck);

	// Body : Create a Hollowed Solid
	TopoDS_Face   faceToRemove;
	Standard_Real zMax = -1;

	for (TopExp_Explorer aFaceExplorer(myBody, TopAbs_FACE); aFaceExplorer.More(); aFaceExplorer.Next()) {
		TopoDS_Face aFace = TopoDS::Face(aFaceExplorer.Current());
		// Check if <aFace> is the top face of the bottles neck 
		Handle(Geom_Surface) aSurface = BRep_Tool::Surface(aFace);
		if (aSurface->DynamicType() == STANDARD_TYPE(Geom_Plane)) {
			Handle(Geom_Plane) aPlane = Handle(Geom_Plane)::DownCast(aSurface);
			gp_Pnt aPnt = aPlane->Location();
			Standard_Real aZ = aPnt.Z();
			if (aZ > zMax) {
				zMax = aZ;
				faceToRemove = aFace;
			}
		}
	}

	TopTools_ListOfShape facesToRemove;
	facesToRemove.Append(faceToRemove);
	myBody = BRepOffsetAPI_MakeThickSolid(myBody, facesToRemove, -myThickness / 50, 1.e-3);
	// Threading : Create Surfaces
	Handle(Geom_CylindricalSurface) aCyl1 = new Geom_CylindricalSurface(neckAx2, myNeckRadius * 0.99);
	Handle(Geom_CylindricalSurface) aCyl2 = new Geom_CylindricalSurface(neckAx2, myNeckRadius * 1.05);

	// Threading : Define 2D Curves
	gp_Pnt2d aPnt(2. * M_PI, myNeckHeight / 2.);
	gp_Dir2d aDir(2. * M_PI, myNeckHeight / 4.);
	gp_Ax2d anAx2d(aPnt, aDir);

	Standard_Real aMajor = 2. * M_PI;
	Standard_Real aMinor = myNeckHeight / 10;

	Handle(Geom2d_Ellipse) anEllipse1 = new Geom2d_Ellipse(anAx2d, aMajor, aMinor);
	Handle(Geom2d_Ellipse) anEllipse2 = new Geom2d_Ellipse(anAx2d, aMajor, aMinor / 4);
	Handle(Geom2d_TrimmedCurve) anArc1 = new Geom2d_TrimmedCurve(anEllipse1, 0, M_PI);
	Handle(Geom2d_TrimmedCurve) anArc2 = new Geom2d_TrimmedCurve(anEllipse2, 0, M_PI);
	gp_Pnt2d anEllipsePnt1 = anEllipse1->Value(0);
	gp_Pnt2d anEllipsePnt2 = anEllipse1->Value(M_PI);

	Handle(Geom2d_TrimmedCurve) aSegment = GCE2d_MakeSegment(anEllipsePnt1, anEllipsePnt2);
	// Threading : Build Edges and Wires
	TopoDS_Edge anEdge1OnSurf1 = BRepBuilderAPI_MakeEdge(anArc1, aCyl1);
	TopoDS_Edge anEdge2OnSurf1 = BRepBuilderAPI_MakeEdge(aSegment, aCyl1);
	TopoDS_Edge anEdge1OnSurf2 = BRepBuilderAPI_MakeEdge(anArc2, aCyl2);
	TopoDS_Edge anEdge2OnSurf2 = BRepBuilderAPI_MakeEdge(aSegment, aCyl2);
	TopoDS_Wire threadingWire1 = BRepBuilderAPI_MakeWire(anEdge1OnSurf1, anEdge2OnSurf1);
	TopoDS_Wire threadingWire2 = BRepBuilderAPI_MakeWire(anEdge1OnSurf2, anEdge2OnSurf2);
	BRepLib::BuildCurves3d(threadingWire1);
	BRepLib::BuildCurves3d(threadingWire2);

	// Create Threading 
	BRepOffsetAPI_ThruSections aTool(Standard_True);
	aTool.AddWire(threadingWire1);
	aTool.AddWire(threadingWire2);
	aTool.CheckCompatibility(Standard_False);

	TopoDS_Shape myThreading = aTool.Shape();

	// Building the Resulting Compound 
	TopoDS_Compound aRes;
	BRep_Builder aBuilder;
	aBuilder.MakeCompound(aRes);
	aBuilder.Add(aRes, myBody);
	aBuilder.Add(aRes, myThreading);

	return aRes;
}



cMainApp::cMainApp()
{
}


cMainApp::~cMainApp()
{
}


void cMainApp::run()
{

	initWindow();
	mainloop();
	cleanup();
}

char *cMainApp::getLastErrorDescr()
{
	return szErrore;
}

long cMainApp::getLastError()
{
	return NumErrore;
}


void cMainApp::initWindow()
{
	NumErrore = 0;
	glfwSetErrorCallback(error_callback);



	glfwInit();

	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
	glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);
//	glfwWindowHint(GLFW_CLIENT_API, GLFW_NO_API);

	IFWin::WinRect.w = 800;
	IFWin::WinRect.h = 600;



	window = glfwCreateWindow(IFWin::WinRect.w, IFWin::WinRect.h, "app", nullptr, nullptr);

	IFWin::ID = glfwGetWin32Window(window);
	IFWin::bVisible = true;
	

	glfwSetWindowUserPointer(window, this);


	// window callback
	glfwSetWindowSizeCallback(window, cMainApp::s_onWindowResized);
	glfwSetWindowPosCallback(window, cMainApp::s_onWindowPos);
	glfwSetWindowCloseCallback(window, cMainApp::s_onWindowClose);
	glfwSetWindowRefreshCallback(window, cMainApp::s_onWindowRefresh);
	glfwSetWindowFocusCallback(window, cMainApp::s_onWindowFocus);
	glfwSetWindowIconifyCallback(window, cMainApp::s_onWindowIconify);
	glfwSetFramebufferSizeCallback(window, cMainApp::s_onFBResized);


	// mouse callback

	glfwSetScrollCallback(window, cMainApp::s_onMouseScroll);
	glfwSetMouseButtonCallback(window, cMainApp::s_onMouseButton);
	glfwSetCursorEnterCallback(window, cMainApp::s_onMouseEnterLeave);
	glfwSetCursorPosCallback(window, cMainApp::s_onMouseMove);


	// keyboard callback

	glfwSetKeyCallback(window, cMainApp::s_onKey);
	glfwSetCharCallback(window, cMainApp::s_onChar);





//	glewExperimental = GL_TRUE;


	glfwMakeContextCurrent(window);
	mGlContext=::wglGetCurrentContext();
	////glewExperimental = true; // Needed in core profile
	if (glewInit() != GLEW_OK) {
		fprintf(stderr, "Failed to initialize GLEW\n");
		return;
	}

	{
		char buggGl[512];
		sprintf_s(buggGl, "OpenGL %s, GLSL %s", glGetString(GL_VERSION),
			glGetString(GL_SHADING_LANGUAGE_VERSION));

		::SetWindowTextA(IFWin::ID, buggGl);
		long i = 1;;

	}
	// Setup ImGui binding
	ImGui_ImplGlfwGL3_Init(window, false);

	// Load Fonts
	// (there is a default font, this is only if you want to change it. see extra_fonts/README.txt for more details)
	//ImGuiIO& io = ImGui::GetIO();
	//io.Fonts->AddFontDefault();
	//io.Fonts->AddFontFromFileTTF("../../extra_fonts/Cousine-Regular.ttf", 15.0f);
	//io.Fonts->AddFontFromFileTTF("../../extra_fonts/DroidSans.ttf", 16.0f);
	//io.Fonts->AddFontFromFileTTF("../../extra_fonts/ProggyClean.ttf", 13.0f);
	//io.Fonts->AddFontFromFileTTF("../../extra_fonts/ProggyTiny.ttf", 10.0f);
	//io.Fonts->AddFontFromFileTTF("c:\\Windows\\Fonts\\ArialUni.ttf", 18.0f, NULL, io.Fonts->GetGlyphRangesJapanese());


	bkColor = ImColor(114, 144, 154);




	initOCC();

	SetTheme(0);

}


bool cMainApp::initOCC()
{
	if (!m_isInitialized) {
		Handle_Graphic3d_GraphicDriver gpxDriver;
		Handle(OpenGl_GraphicDriver) myGraphicDriver;
		Handle_Aspect_DisplayConnection dispConnection;


		if (dispConnection.IsNull())
			dispConnection = new Aspect_DisplayConnection;




		myGraphicDriver = new OpenGl_GraphicDriver(dispConnection,false);
		myGraphicDriver->SetBuffersNoSwap(true);

		//OpenGl_Caps& aCaps = myGraphicDriver->ChangeOptions();
		//aCaps.contextStereo = Standard_True;




		// Create the named OCC 3d viewer
		m_viewer = new V3d_Viewer(myGraphicDriver/*, reinterpret_cast<Standard_ExtString>("Viewer3d")*/);

		// Configure the OCC 3d viewer
		m_viewer->SetDefaultViewSize(1000.);
		m_viewer->SetDefaultViewProj(V3d_XposYnegZpos);
		m_viewer->SetDefaultBackgroundColor(Quantity_NOC_BLACK);
		m_viewer->SetDefaultVisualization(V3d_ZBUFFER);
		m_viewer->SetDefaultShadingModel(V3d_GOURAUD);

		m_viewer->SetUpdateMode(V3d_WAIT);


		m_viewer->SetDefaultLights();
		m_viewer->SetLightOn();




		m_GLcontext = new OpenGl_Context();
		m_GLcontext->Init(IFWin::ID, ::GetDC(IFWin::ID), mGlContext);
		//m_GLcontext->Init(true);



		//////////////////////////////////////////


		m_view = m_viewer->CreateView();

		Handle_OcctWindow hWnd = new OcctWindow(this);


		m_view->SetWindow(hWnd, mGlContext);


		if (!hWnd->IsMapped())
			hWnd->Map();




		m_view->SetBgGradientColors(
			Quantity_Color(0.25, 0.28, 0.75, Quantity_TOC_RGB),
			Quantity_Color(0.10, 0.0, 0.10, Quantity_TOC_RGB),
//			Quantity_NOC_WHITE,
			Aspect_GFM_VER);






		m_view->TriedronDisplay(
			Aspect_TOTP_LEFT_LOWER,
			Quantity_NOC_GRAY50,
			0.075,
			V3d_ZBUFFER);






		m_view->MustBeResized();
		m_isInitialized = true;
		m_needsResize = true;


		



		mContext = new AIS_InteractiveContext(m_viewer);
		mContext->SetDisplayMode(AIS_Shaded);

		mContext->MainSelector()->SetPickClosest(Standard_False);


		TopoDS_Shape aBottle;
		aBottle = MakeBottle(150, 270, 130);
		//	aBottle2 = MakeBottle(50, 120, 30);

		//aBottle2.Move(TopLoc_Location())
		Handle(AIS_Shape) AISBottle = new AIS_Shape(aBottle);
		mContext->SetMaterial(AISBottle, Graphic3d_NOM_GOLD);
		mContext->SetDisplayMode(AISBottle, 1, Standard_False);
		mContext->Display(AISBottle, Standard_False);
		
		mContext->SetCurrentObject(AISBottle, Standard_False);
		
		
		//TopoDS_Shape aShape = BRepPrimAPI_MakeBox(10, 20, 30).Solid();
		//Handle(AIS_Shape) anAISShape = new AIS_Shape(aShape);
		//mContext->Display(anAISShape);


		//m_GLcontext->SwapBuffers();
		mCamera = m_view->Camera();

		mManipulator= new AIS_Manipulator();
		//mManipulator->SetPart(0, AIS_ManipulatorMode::AIS_MM_Scaling, Standard_False);
		//mManipulator->SetPart(1, AIS_ManipulatorMode::AIS_MM_Rotation, Standard_False);

		//AISBottle->SetPickClosest(Standard_False)
		 
		

		//mManipulator->EnableMode(AIS_ManipulatorMode::AIS_MM_Translation);
		//mManipulator->EnableMode(AIS_ManipulatorMode::AIS_MM_Scaling);
		//mManipulator->EnableMode(AIS_ManipulatorMode::AIS_MM_Rotation);
		mManipulator->SetModeActivationOnDetection(Standard_True);
		//mManipulator->SetSize(50);

	}

	return true;
}



void cMainApp::cleanup()
{
	glfwDestroyWindow(window);
	glfwTerminate();
}



const Handle_V3d_Viewer& cMainApp::occV3dViewer() const
{
	return m_viewer;
}


void cMainApp::setOccV3dViewer(const Handle_V3d_Viewer& viewer)
{
	//ASSERTA(viewer.IsNull());
	m_viewer = viewer;
}

const Handle_V3d_View& cMainApp::occV3dView() const
{
	return m_view;
}



void cMainApp::s_onWindowResized(GLFWwindow* window, int width, int height)
{
	if (width == 0 || height == 0) return;

	cMainApp* app = reinterpret_cast<cMainApp*>(glfwGetWindowUserPointer(window));
	app->OnWindowResized(width,height);
}

void cMainApp::s_onFBResized(GLFWwindow* window, int width, int height)
{
	if (width == 0 || height == 0) return;

	cMainApp* app = reinterpret_cast<cMainApp*>(glfwGetWindowUserPointer(window));
	app->OnFBResized(width, height);
}

void cMainApp::s_onWindowPos(GLFWwindow* window, int xpos, int ypos)
{
	

	cMainApp* app = reinterpret_cast<cMainApp*>(glfwGetWindowUserPointer(window));
	app->OnWindowPos(xpos,ypos);
}


void cMainApp::s_onWindowIconify(GLFWwindow* window, int iconified)
{


	cMainApp* app = reinterpret_cast<cMainApp*>(glfwGetWindowUserPointer(window));
	app->OnWindowIconify(iconified== GLFW_TRUE);
}


void cMainApp::s_onWindowFocus(GLFWwindow* window, int focused)
{


	cMainApp* app = reinterpret_cast<cMainApp*>(glfwGetWindowUserPointer(window));
	app->OnWindowFocus(focused == GLFW_TRUE);
}



void cMainApp::s_onWindowClose(GLFWwindow* window)
{


	cMainApp* app = reinterpret_cast<cMainApp*>(glfwGetWindowUserPointer(window));
	app->OnWindowClose();
}

void cMainApp::s_onWindowRefresh(GLFWwindow* window)
{


	cMainApp* app = reinterpret_cast<cMainApp*>(glfwGetWindowUserPointer(window));
	app->OnWindowRefresh();
}



void cMainApp::s_onMouseButton(GLFWwindow* window, int button, int action, int mods)
{
	cMainApp* app = reinterpret_cast<cMainApp*>(glfwGetWindowUserPointer(window));
	app->OnMouseButton(button, action, mods);
}

void cMainApp::s_onMouseMove(GLFWwindow* window, double xpos, double ypos)
{
	cMainApp* app = reinterpret_cast<cMainApp*>(glfwGetWindowUserPointer(window));
	app->OnMouseMove(xpos, ypos);
}
void cMainApp::s_onMouseScroll(GLFWwindow* window, double xoffset, double yoffset)
{
	cMainApp* app = reinterpret_cast<cMainApp*>(glfwGetWindowUserPointer(window));
	app->OnMouseScroll(xoffset, yoffset);
}
void cMainApp::s_onMouseEnterLeave(GLFWwindow* window, int entered)
{
	cMainApp* app = reinterpret_cast<cMainApp*>(glfwGetWindowUserPointer(window));
	app->OnMouseEnterLeave(entered);

}

void cMainApp::s_onKey(GLFWwindow* window, int key, int scancode, int action, int mods)
{
	cMainApp* app = reinterpret_cast<cMainApp*>(glfwGetWindowUserPointer(window));
	app->OnKey(key, scancode, action, mods);

}

void cMainApp::s_onChar(GLFWwindow* window, unsigned int c)
{
	cMainApp* app = reinterpret_cast<cMainApp*>(glfwGetWindowUserPointer(window));
	app->OnChar(c);
	
}


void cMainApp::OnKey(int key, int scancode, int action, int mods)
{
	ImGui_ImplGlfwGL3_KeyCallback(window, key, scancode, action, mods);
}

void cMainApp::OnChar(int c)
{
	ImGui_ImplGlfwGL3_CharCallback(window, c);
}


void cMainApp::OnMouseMove(double xpos, double ypos)
{

	

	//if (dragging)
	//{
	//	if (!m_view.IsNull())
	//	{

	//		//m_view->Rotate(xpos, ypos);
	//		m_view->SetImmediateUpdate(false);
	//		m_view->Rotation(xpos, ypos);
	//		m_view->SetImmediateUpdate(true);

	//		//m_view->Rotation((int)xpos - mouseX, (int)ypos - mouseY);
	//		//m_view->Panning(-(xpos - mouseX)/100.0, -(ypos - mouseY)/100.0);


	//	}

	//}


	//double xpos, ypos;
	//glfwGetCursorPos(window, &xpos, &ypos);
	mouseX = (int)xpos;
	mouseY = (int)ypos;

	if (mManipulator->HasActiveMode())
	{
		::OutputDebugString(L"\nmManipulator->HasActiveMode()");
		mManipulator->Transform(mouseX, mouseY, m_view);
		m_view->Redraw();
	}


}

void cMainApp::OnMouseScroll(double xoffset, double yoffset)
{
	ImGui_ImplGlfwGL3_ScrollCallback(window, xoffset, yoffset);
}

void cMainApp::OnMouseButton(int button, int action, int mods)
{

	double xpos, ypos;
	glfwGetCursorPos(window, &xpos, &ypos);
	//mouseX = (int)xpos;
	//mouseY = (int)ypos;
	ButtonMouseX = (int)xpos;
	ButtonMouseY = (int)ypos;


	if (button == GLFW_MOUSE_BUTTON_LEFT)
	{


		if (action == GLFW_PRESS)
		{

			
			mContext->MainSelector()->Pick(ButtonMouseX, ButtonMouseY, m_view);

			mouseX = mouseX;

			//mContext->MainSelector()->Init();

			long nSelect = mContext->MainSelector()->NbPicked();

			

			if (nSelect > 0)
			{
				Handle_SelectMgr_SelectableObject obj = mContext->MainSelector()->Picked(1)->Selectable();
				
				Handle_AIS_Shape aisShp = Handle_AIS_Shape::DownCast(obj);
				aisShp->SetTransparency(0.2);

				if (!aisShp.IsNull() && !aisShp->IsKind(STANDARD_TYPE(AIS_Manipulator)))
				{

					AIS_Manipulator::OptionsForAttach anOptions;
					anOptions.SetAdjustPosition(Standard_True);
					anOptions.SetAdjustSize(Standard_True);
					anOptions.SetEnableModes(Standard_False);

				
					//mManipulator->SetPart(0, AIS_ManipulatorMode::AIS_MM_Scaling, Standard_False);
					//mManipulator->SetPart(1, AIS_ManipulatorMode::AIS_MM_Rotation, Standard_False);
					////mManipulator->EnableMode(AIS_ManipulatorMode::AIS_MM_Scaling);
					////mManipulator->EnableMode(AIS_ManipulatorMode::AIS_MM_Rotation);
					mManipulator->SetModeActivationOnDetection(Standard_True);

					mManipulator->Attach(aisShp, anOptions);

					//mManipulator->EnableMode(AIS_ManipulatorMode::AIS_MM_Translation);


					if (mManipulator->HasActiveMode())
					{
						
						nSelect = nSelect;
					}
				}


			}
			else
			{
				mContext->MainSelector()->Clear();
				mManipulator->Detach();

			}

		}
		else if (action == GLFW_RELEASE)
		{


		}

	}
	else
		if (button == GLFW_MOUSE_BUTTON_RIGHT)
		{


			if (action == GLFW_PRESS)
			{
				//dragging = true;

				mContext->MainSelector()->Pick(ButtonMouseX, ButtonMouseY, this->m_view);
				long nSelect = mContext->MainSelector()->NbPicked();


				if (mManipulator->HasActiveMode())

				{
					mManipulator->StartTransform(ButtonMouseX, ButtonMouseY, m_view);
				}

			}
			else if (action == GLFW_RELEASE)
			{

				mManipulator->StopTransform();


			}

		}


	ImGui_ImplGlfwGL3_MouseButtonCallback(window, button, action, mods);
}


void cMainApp::OnMouseEnterLeave(int entered)
{
	if (entered)
	{
		// The cursor entered the client area of the window
	}
	else
	{
		// The cursor left the client area of the window
	}
}


void cMainApp::OnError(int error, const char* description)
{
}









void cMainApp::mainloop()
{
	while (!glfwWindowShouldClose(window)) {
		glfwPollEvents();

		//		updateUniformBuffer();
		drawFrame();
	}
}



void cMainApp::OnWindowResized(int width, int height)
{
	IFWin::WinRect.w=width; IFWin::WinRect.h = height;

	if (!m_view.IsNull())
	{
		m_view->Window()->DoResize();
	}
	m_needsResize = true;

}

void cMainApp::OnWindowPos(int xPos, int yPos)
{
	IFWin::WinRect.x = xPos; IFWin::WinRect.y = yPos;

}

void cMainApp::OnWindowClose()
{
}

void cMainApp::OnWindowRefresh()
{

}

void cMainApp::OnWindowFocus(bool focused)
{
}

void cMainApp::OnWindowIconify(bool iconified)
{
	IFWin::bVisible = !iconified;
}

void cMainApp::OnFBResized(int width, int height)
{
	IFWin::WinRect.w = width; IFWin::WinRect.h = height;
	if (!m_view.IsNull())
	{
		m_view->Window()->DoResize();
	}
	m_needsResize = true;
}




void cMainApp::redraw()
{
	if (!m_view.IsNull())
	{

		m_GLcontext->MakeCurrent();

		if (m_needsResize)
			m_view->MustBeResized();
		else
		{
			//render();
			//Refresh();
				m_view->Invalidate();



		}


	}

	m_needsResize = false;
}

void cMainApp::fitAll()
{
	if (!m_view.IsNull()) {
		m_view->ZFitAll();
		m_view->FitAll();
		redraw();
	}
}


void cMainApp::drawFrame()
{
	bool show_test_window = true;
	bool show_another_window = false;




	if (!m_view.IsNull())
	{

		m_GLcontext->MakeCurrent();

		if (m_needsResize)
			m_view->MustBeResized();
		//else
		{
			m_view->Invalidate();



		}
	}


	ImGui_ImplGlfwGL3_NewFrame();

	// 1. Show a simple window
	// Tip: if we don't call ImGui::Begin()/ImGui::End() the widgets appears in a window automatically called "Debug"
	{
		static float f = 0.0f;
		ImGui::Text("Hello, world!");
		ImGui::SliderFloat("float", &f, 0.0f, 1.0f);
		ImGui::ColorEdit3("clear color", (float*)&bkColor);
		if (ImGui::Button("Test Window")) show_test_window ^= 1;
		if (ImGui::Button("Another Window")) show_another_window ^= 1;
		ImGui::Text("Application average %.3f \nms/frame (%.1f FPS)", 1000.0f / ImGui::GetIO().Framerate, ImGui::GetIO().Framerate);
	}

	// 2. Show another simple window, this time using an explicit Begin/End pair
	//if (show_another_window)
	{
		//ImGui::Begin("finHello");
		ImGui::SetNextWindowSize(ImVec2(200, 200), ImGuiSetCond_FirstUseEver);
		ImGui::Begin("Another Window", &show_another_window);
		ImGui::Text("Hello");
		ImGui::End();
	}

	// 3. Show the ImGui test window. Most of the sample code is in ImGui::ShowTestWindow()
	if (show_test_window)
	{
		ImGui::SetNextWindowPos(ImVec2(650, 20), ImGuiSetCond_FirstUseEver);
		ImGui::ShowTestWindow(&show_test_window);
	}


	ImGuizmo::BeginFrame();
	ImGuizmo::Enable(true);

	ImGuiIO& io = ImGui::GetIO();
	ImGuizmo::SetRect(0, 0, io.DisplaySize.x, io.DisplaySize.y);

	//ModelWorldState = { myStack = { ... } myCurrent = { myMat = 0x073c3928 {1.00000000, 0.000000000, 0.000000000, 0.000000000, 0.000000000, ...} } ... }


	//OpenGl::MatrixType<Standard_ShortReal>::Mat4 mw = m_GLcontext->ModelWorldState.Current();
	
	Handle(Graphic3d_Camera) aCamera = this->m_view->Camera();
	
	OpenGl_Mat4d mproject = aCamera->ProjectionMatrix();

	float  proj[16] = { 0 };
	float  view[16] = { 0 };
	float  matRes[16] = { 0 };

	proj[0] = proj[5] = proj[10] = proj[15] = 1;
	view[0] = view[5] = view[10] = view[15] = 1;

	//memcpy(proj, aCamera->ProjectionState()., 16 * sizeof(float));
	//memcpy(view, this->mContext->CurrentViewer(), 16 * sizeof(float));
	gp_Dir pUp = aCamera->Up();
	gp_Pnt pEye= aCamera->Eye();
	gp_Pnt pCenter = aCamera->Center();


	geoMatrix4 mProj;
	
	memcpy(matRes, mProj.pointerAsFloat(), 16 * sizeof(float));
	memcpy(view, mProj.pointerAsFloat(), 16 * sizeof(float));
	mProj.buildCameraLookAtMatrixLH(geoVector3D(pEye.X(), pEye.Y(), pEye.Z()),
									geoVector3D(pCenter.X(), pCenter.Y(), pCenter.Z()),
									geoVector3D(pUp.X(), pUp.Y(), pUp.Z()));


	ImGuizmo::Manipulate(view, mProj.pointerAsFloat(),ImGuizmo::TRANSLATE,ImGuizmo::WORLD, matRes);

	// Rendering
	int display_w, display_h;
	glfwGetFramebufferSize(window, &display_w, &display_h);
	glViewport(0, 0, display_w, display_h);
	//glClearColor(bkColor.x, bkColor.y, bkColor.z, bkColor.w);
	//glClear(GL_COLOR_BUFFER_BIT);











	ImGui::Render();
	//glfwSwapBuffers(window);
	m_GLcontext->SwapBuffers();
	


	
}




void cMainApp::SetTheme(long nTema)
{
	ImGuiStyle& style = ImGui::GetStyle();
	style.WindowRounding = 0.0f;
	style.FrameRounding = 0.0f;
	style.ScrollbarRounding = 0;

	style.Colors[ImGuiCol_Text] = ImVec4(0.90f, 0.90f, 0.90f, 0.90f);
	style.Colors[ImGuiCol_TextDisabled] = ImVec4(0.60f, 0.60f, 0.60f, 1.00f);
	style.Colors[ImGuiCol_WindowBg] = ImVec4(0.09f, 0.09f, 0.15f, 1.00f);
	style.Colors[ImGuiCol_ChildWindowBg] = ImVec4(0.00f, 0.00f, 0.00f, 0.00f);
	style.Colors[ImGuiCol_PopupBg] = ImVec4(0.05f, 0.05f, 0.10f, 0.85f);
	style.Colors[ImGuiCol_Border] = ImVec4(0.70f, 0.70f, 0.70f, 0.65f);
	style.Colors[ImGuiCol_BorderShadow] = ImVec4(0.00f, 0.00f, 0.00f, 0.00f);
	style.Colors[ImGuiCol_FrameBg] = ImVec4(0.00f, 0.00f, 0.01f, 1.00f);
	style.Colors[ImGuiCol_FrameBgHovered] = ImVec4(0.90f, 0.80f, 0.80f, 0.40f);
	style.Colors[ImGuiCol_FrameBgActive] = ImVec4(0.90f, 0.65f, 0.65f, 0.45f);
	style.Colors[ImGuiCol_TitleBg] = ImVec4(0.70f, 0.00f, 0.00f, 0.83f);
	style.Colors[ImGuiCol_TitleBgCollapsed] = ImVec4(0.40f, 0.40f, 0.80f, 0.20f);
	style.Colors[ImGuiCol_TitleBgActive] = ImVec4(0.90f, 0.00f, 0.00f, 0.87f);
	style.Colors[ImGuiCol_MenuBarBg] = ImVec4(0.01f, 0.01f, 0.02f, 0.80f);
	style.Colors[ImGuiCol_ScrollbarBg] = ImVec4(0.20f, 0.25f, 0.30f, 0.60f);
	style.Colors[ImGuiCol_ScrollbarGrab] = ImVec4(0.55f, 0.53f, 0.55f, 0.51f);
	style.Colors[ImGuiCol_ScrollbarGrabHovered] = ImVec4(0.56f, 0.56f, 0.56f, 1.00f);
	style.Colors[ImGuiCol_ScrollbarGrabActive] = ImVec4(0.56f, 0.56f, 0.56f, 0.91f);
	style.Colors[ImGuiCol_ComboBg] = ImVec4(0.1f, 0.1f, 0.1f, 0.99f);
	style.Colors[ImGuiCol_CheckMark] = ImVec4(0.90f, 0.90f, 0.90f, 0.83f);
	style.Colors[ImGuiCol_SliderGrab] = ImVec4(0.70f, 0.70f, 0.70f, 0.62f);
	style.Colors[ImGuiCol_SliderGrabActive] = ImVec4(0.30f, 0.30f, 0.30f, 0.84f);
	style.Colors[ImGuiCol_Button] = ImVec4(0.48f, 0.72f, 0.89f, 0.49f);
	style.Colors[ImGuiCol_ButtonHovered] = ImVec4(0.50f, 0.69f, 0.99f, 0.68f);
	style.Colors[ImGuiCol_ButtonActive] = ImVec4(0.80f, 0.50f, 0.50f, 1.00f);
	style.Colors[ImGuiCol_Header] = ImVec4(0.30f, 0.69f, 1.00f, 0.53f);
	style.Colors[ImGuiCol_HeaderHovered] = ImVec4(0.44f, 0.61f, 0.86f, 1.00f);
	style.Colors[ImGuiCol_HeaderActive] = ImVec4(0.38f, 0.62f, 0.83f, 1.00f);
	style.Colors[ImGuiCol_Column] = ImVec4(0.50f, 0.50f, 0.50f, 1.00f);
	style.Colors[ImGuiCol_ColumnHovered] = ImVec4(0.70f, 0.60f, 0.60f, 1.00f);
	style.Colors[ImGuiCol_ColumnActive] = ImVec4(0.90f, 0.70f, 0.70f, 1.00f);
	style.Colors[ImGuiCol_ResizeGrip] = ImVec4(1.00f, 1.00f, 1.00f, 0.85f);
	style.Colors[ImGuiCol_ResizeGripHovered] = ImVec4(1.00f, 1.00f, 1.00f, 0.60f);
	style.Colors[ImGuiCol_ResizeGripActive] = ImVec4(1.00f, 1.00f, 1.00f, 0.90f);
	style.Colors[ImGuiCol_CloseButton] = ImVec4(0.50f, 0.50f, 0.90f, 0.50f);
	style.Colors[ImGuiCol_CloseButtonHovered] = ImVec4(0.70f, 0.70f, 0.90f, 0.60f);
	style.Colors[ImGuiCol_CloseButtonActive] = ImVec4(0.70f, 0.70f, 0.70f, 1.00f);
	style.Colors[ImGuiCol_PlotLines] = ImVec4(1.00f, 1.00f, 1.00f, 1.00f);
	style.Colors[ImGuiCol_PlotLinesHovered] = ImVec4(0.90f, 0.70f, 0.00f, 1.00f);
	style.Colors[ImGuiCol_PlotHistogram] = ImVec4(0.90f, 0.70f, 0.00f, 1.00f);
	style.Colors[ImGuiCol_PlotHistogramHovered] = ImVec4(1.00f, 0.60f, 0.00f, 1.00f);
	style.Colors[ImGuiCol_TextSelectedBg] = ImVec4(0.00f, 0.00f, 1.00f, 0.35f);
	style.Colors[ImGuiCol_ModalWindowDarkening] = ImVec4(0.20f, 0.20f, 0.20f, 0.35f);
}

